'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createModel;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Types = require('./Types');

var _Types2 = _interopRequireDefault(_Types);

var _isModel = require('./isModel');

var _isModel2 = _interopRequireDefault(_isModel);

var _isCollection = require('./isCollection');

var _isCollection2 = _interopRequireDefault(_isCollection);

var _Method = require('./errors/Method');

var _Method2 = _interopRequireDefault(_Method);

var _Model = require('./base/Model');

var _Model2 = _interopRequireDefault(_Model);

var _events = require('./mixins/events');

var _events2 = _interopRequireDefault(_events);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function createModel() {
  var schema = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var methods = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var Model = function (_BaseModel) {
    _inherits(Model, _BaseModel);

    function Model() {
      var givenAttributes = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      _classCallCheck(this, Model);

      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Model).call(this, givenAttributes));

      var self = _this;

      var attributes = {};

      // others listening to this
      var listeners = {};

      // apply mixins
      (0, _events2.default)(_this, listeners);

      // built-in methods
      Object.defineProperty(_this, 'toJS', {
        value: function value() {
          function convertToJS(attrs) {
            return _lodash2.default.mapValues(attrs, function (v, k) {
              if ((0, _isModel2.default)(v) || (0, _isCollection2.default)(v)) {
                return v.toJS();
              }

              if (_lodash2.default.isPlainObject(v)) {
                return convertToJS(v);
              }

              return v;
            });
          }

          return convertToJS(attributes);
        }
      });

      Object.defineProperty(_this, 'destroy', {
        value: function value() {
          this.trigger('destroy');
          this.off();

          _lodash2.default.each(attributes, function (v, k) {
            if ((0, _isModel2.default)(v) || (0, _isCollection2.default)(v)) {
              v.destroy();
            }
          });
        }
      });

      // parse by schema
      var applySchema = _Types2.default.object.of(schema);
      attributes = applySchema(givenAttributes);

      // define attributes
      _lodash2.default.each(attributes, function (value, attributeName) {
        Object.defineProperty(_this, attributeName, {
          get: function get() {
            return attributes[attributeName];
          },
          set: function set(newValue) {
            if (schema[attributeName](newValue)) {
              attributes[attributeName] = newValue;

              self.trigger('change');
            }
          },


          enumerable: true
        });

        // watch children
        if ((0, _isModel2.default)(value)) {
          (function () {
            var watcher = value.on('change', function () {
              self.trigger('change');
            });

            value.on('destroy', function () {
              self.trigger('change');
              watcher();
            });
          })();
        }
      });

      // define methods
      _lodash2.default.each(methods, function (func, methodName) {
        if (typeof attributes[methodName] !== 'undefined' || typeof _this[methodName] !== 'undefined') {
          throw new _Method2.default('conflicting method name: ' + methodName);
        }

        _this[methodName] = func.bind(_this);
      });
      return _this;
    }

    return Model;
  }(_Model2.default);

  return Model;
}